@@ -15,22 +15,21 @@
  */
 
 import { defineMessages } from 'react-intl';
-import { normalize } from 'normalizr';
 import when from 'when';
 import yaml from 'js-yaml';
+import { startSubmit, stopSubmit } from 'redux-form';
 
 import { handleErrors } from './ErrorActions';
 import history from '../utils/history';
 import MistralApiService from '../services/MistralApiService';
 import NotificationActions from '../actions/NotificationActions';
 import PlansConstants from '../constants/PlansConstants';
-import { planFileSchema } from '../normalizrSchemas/plans';
-import StackActions from '../actions/StacksActions';
 import SwiftApiService from '../services/SwiftApiService';
 import MistralConstants from '../constants/MistralConstants';
 import { PLAN_ENVIRONMENT } from '../constants/PlansConstants';
 import { getServiceUrl } from '../selectors/auth';
 import { startWorkflow } from './WorkflowActions';
+import { sanitizeMessage } from '../utils';
 
 const messages = defineMessages({
   planCreatedNotificationTitle: {
@@ -41,6 +40,10 @@ const messages = defineMessages({
     id: 'PlansActions.planCreatedNotificationMessage',
     defaultMessage: 'The plan {planName} was successfully created.'
   },
+  planCreationFailed: {
+    id: 'PlansActions.planCreationFailedNotificationMessage',
+    defaultMessage: 'Plan creation failed'
+  },
   planUpdatedNotificationTitle: {
     id: 'PlansActions.planUpdatedNotificationTitle',
     defaultMessage: 'Plan Updated'
@@ -61,10 +64,6 @@ const messages = defineMessages({
     id: 'PlansActions.planDeletedNotificationMessage',
     defaultMessage: 'The plan {planName} was successfully deleted.'
   },
-  deploymentFailedNotificationTitle: {
-    id: 'PlansActions.deploymentFailedNotificationTitle',
-    defaultMessage: 'Deployment Failed'
-  },
   exportFailedNotificationTitle: {
     id: 'PlansActions.exportFailedNotificationTitle',
     defaultMessage: 'Export Failed'
@@ -72,85 +71,100 @@ const messages = defineMessages({
 });
 
 export default {
-  requestPlans() {
+  fetchPlansPending() {
     return {
-      type: PlansConstants.REQUEST_PLANS
+      type: PlansConstants.FETCH_PLANS_PENDING
     };
   },
 
-  receivePlans(plans) {
+  fetchPlansSuccess(plans) {
     return {
-      type: PlansConstants.RECEIVE_PLANS,
+      type: PlansConstants.FETCH_PLANS_SUCCESS,
       payload: plans
     };
   },
 
   fetchPlans() {
     return dispatch => {
-      dispatch(this.requestPlans());
-      return (
-        dispatch(MistralApiService.runAction(MistralConstants.PLAN_LIST))
-          // TODO(jtomasek): This block should be done on Mistral action side
-          .then(planNames =>
-            when
-              .all(
-                planNames.map(name =>
-                  dispatch(SwiftApiService.getObject(name, PLAN_ENVIRONMENT))
-                )
-              )
-              .then(planEnvs =>
-                planNames.map(planName => {
-                  for (let i = 0; i < planEnvs.length; i++) {
-                    const { name, description } = yaml.safeLoad(planEnvs[i]);
-                    if (name === planName) {
-                      return { name, description };
-                    }
-                  }
-                  return { name: planName };
-                })
-              )
-              .catch(error => {
-                dispatch(
-                  handleErrors(error, 'Plan descriptions could not be loaded')
-                );
-                return planNames.map(name => ({ name }));
-              })
-          )
-          .then(plans => dispatch(this.receivePlans(plans)))
-          .catch(error =>
-            dispatch(handleErrors(error, 'Plans could not be loaded'))
-          )
-      );
+      dispatch(this.fetchPlansPending());
+      return dispatch(MistralApiService.runAction(MistralConstants.PLAN_LIST))
+        .then(plans => dispatch(this.fetchPlansSuccess(plans)))
+        .catch(error =>
+          dispatch(handleErrors(error, 'Plans could not be loaded'))
+        );
     };
   },
 
-  requestPlan() {
+  fetchPlanFilesPending(planName) {
     return {
-      type: PlansConstants.REQUEST_PLAN
+      type: PlansConstants.FETCH_PLAN_FILES_PENDING,
+      payload: planName
     };
   },
 
-  receivePlan(planName, planFiles) {
+  fetchPlanFilesSuccess(planName, planFiles) {
     return {
-      type: PlansConstants.RECEIVE_PLAN,
-      payload: {
-        planName: planName,
-        planFiles: planFiles
-      }
+      type: PlansConstants.FETCH_PLAN_FILES_SUCCESS,
+      payload: { planName, planFiles }
     };
   },
 
-  fetchPlan(planName) {
+  fetchPlanFilesFailed(planName) {
+    return {
+      type: PlansConstants.FETCH_PLAN_FILES_FAILED,
+      payload: planName
+    };
+  },
+
+  fetchPlanFiles(planName) {
     return dispatch => {
-      dispatch(this.requestPlan());
+      dispatch(this.fetchPlanFilesPending(planName));
       return dispatch(SwiftApiService.getContainer(planName))
         .then(response => {
-          const planFiles =
-            normalize(response, [planFileSchema]).entities.planFiles || {};
-          dispatch(this.receivePlan(planName, planFiles));
+          dispatch(this.fetchPlanFilesSuccess(planName, response));
         })
         .catch(error => {
           dispatch(handleErrors(error, 'Plan could not be loaded'));
+          dispatch(this.fetchPlanFilesFailed(planName));
+        });
+    };
+  },
+
+  fetchPlanDetailsPending(planName) {
+    return {
+      type: PlansConstants.FETCH_PLAN_DETAILS_PENDING,
+      payload: planName
+    };
+  },
+
+  fetchPlanDetailsSuccess(planName, planEnvironment) {
+    return {
+      type: PlansConstants.FETCH_PLAN_DETAILS_SUCCESS,
+      payload: { planName, planEnvironment }
+    };
+  },
+
+  fetchPlanDetailsFailed(planName) {
+    return {
+      type: PlansConstants.FETCH_PLAN_DETAILS_FAILED,
+      payload: planName
+    };
+  },
+
+  fetchPlanDetails(planName) {
+    return dispatch => {
+      dispatch(this.fetchPlanDetailsPending(planName));
+      return dispatch(SwiftApiService.getObject(planName, PLAN_ENVIRONMENT))
+        .then(response =>
+          dispatch(
+            this.fetchPlanDetailsSuccess(planName, yaml.safeLoad(response))
+          )
+        )
+        .catch(error => {
+          if (error.response && error.response.status !== 404) {
+            dispatch(handleErrors(error, 'Plan details could not be loaded'));
+          }
+          dispatch(this.fetchPlanDetailsFailed(planName));
         });
     };
   },
@@ -169,19 +183,17 @@ export default {
     };
   },
 
-  updatePlanFailed(planName, errors) {
+  updatePlanFailed(planName) {
     return {
       type: PlansConstants.UPDATE_PLAN_FAILED,
-      payload: {
-        planName,
-        errors
-      }
+      payload: planName
     };
   },
 
   updatePlan(planName, planFiles) {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
+      dispatch(startSubmit('editPlanForm'));
       dispatch(this.updatePlanPending(planName));
       return dispatch(uploadFilesToContainer(planName, planFiles))
         .then(response =>
@@ -197,14 +209,14 @@ export default {
           )
         )
         .catch(error => {
-          dispatch(handleErrors(error, 'Plan update failed', false));
-          dispatch(
-            this.updatePlanFailed(planName, [
-              {
+          dispatch(this.updatePlanFailed(planName));
+          return dispatch(
+            stopSubmit('editPlanForm', {
+              _error: {
                 title: formatMessage(messages.planUpdateFailed),
                 message: error.message
               }
-            ])
+            })
           );
         });
     };
@@ -213,6 +225,7 @@ export default {
   updatePlanFromTarball(planName, file) {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
+      dispatch(startSubmit('editPlanForm'));
       dispatch(this.updatePlanPending(planName));
       return dispatch(SwiftApiService.uploadTarball(planName, file))
         .then(response => {
@@ -228,14 +241,14 @@ export default {
           );
         })
         .catch(error => {
-          dispatch(handleErrors(error, 'Plan update failed', false));
-          dispatch(
-            this.updatePlanFailed(planName, [
-              {
+          dispatch(this.updatePlanFailed(planName));
+          return dispatch(
+            stopSubmit('editPlanForm', {
+              _error: {
                 title: formatMessage(messages.planUpdateFailed),
                 message: error.message
               }
-            ])
+            })
           );
         });
     };
@@ -263,43 +276,30 @@ export default {
         dispatch(this.fetchPlans());
         history.push('/plans/manage');
       } else {
-        dispatch(
-          this.updatePlanFailed(planName, [
-            { title: formatMessage(messages.planUpdateFailed), message }
-          ])
+        dispatch(this.updatePlanFailed(planName));
+        return dispatch(
+          stopSubmit('editPlanForm', {
+            _error: {
+              title: formatMessage(messages.planUpdateFailed),
+              message: sanitizeMessage(message)
+            }
+          })
         );
       }
     };
   },
 
-  cancelCreatePlan() {
+  createPlanSuccess(planName) {
     return {
-      type: PlansConstants.CANCEL_CREATE_PLAN
-    };
-  },
-
-  createPlanPending() {
-    return {
-      type: PlansConstants.CREATE_PLAN_PENDING
-    };
-  },
-
-  createPlanSuccess() {
-    return {
-      type: PlansConstants.CREATE_PLAN_SUCCESS
-    };
-  },
-
-  createPlanFailed(errors) {
-    return {
-      type: PlansConstants.CREATE_PLAN_FAILED,
-      payload: errors
+      type: PlansConstants.CREATE_PLAN_SUCCESS,
+      payload: planName
     };
   },
 
   createPlan(planName, planFiles) {
-    return dispatch => {
-      dispatch(this.createPlanPending());
+    return (dispatch, getState, { getIntl }) => {
+      const { formatMessage } = getIntl(getState());
+      dispatch(startSubmit('newPlanForm'));
       return dispatch(
         MistralApiService.runAction(MistralConstants.CREATE_CONTAINER, {
           container: planName
@@ -319,19 +319,22 @@ export default {
           )
         )
         .catch(error => {
-          dispatch(handleErrors(error, 'Plan creation failed', false));
           dispatch(
-            this.createPlanFailed([
-              { title: 'Plan creation failed', message: error.message }
-            ])
+            stopSubmit('newPlanForm', {
+              _error: {
+                title: formatMessage(messages.planCreationFailed),
+                message: error.message
+              }
+            })
           );
         });
     };
   },
 
   createPlanFromTarball(planName, file) {
-    return dispatch => {
-      dispatch(this.createPlanPending());
+    return (dispatch, getState, { getIntl }) => {
+      const { formatMessage } = getIntl(getState());
+      dispatch(startSubmit('newPlanForm'));
       dispatch(
         MistralApiService.runAction(MistralConstants.CREATE_CONTAINER, {
           container: planName
@@ -353,16 +356,45 @@ export default {
           )
         )
         .catch(error => {
-          dispatch(handleErrors(error, 'Plan creation failed', false));
           dispatch(
-            this.createPlanFailed([
-              { title: 'Plan creation failed', message: error.message }
-            ])
+            stopSubmit('newPlanForm', {
+              _error: {
+                title: formatMessage(messages.planCreationFailed),
+                message: error.message
+              }
+            })
           );
         });
     };
   },
 
+  createDefaultPlan(planName) {
+    return (dispatch, getState, { getIntl }) => {
+      const { formatMessage } = getIntl(getState());
+      dispatch(startSubmit('newPlanForm'));
+      return dispatch(
+        startWorkflow(
+          MistralConstants.PLAN_CREATE,
+          {
+            container: planName,
+            use_default_templates: true
+          },
+          execution => dispatch(this.createPlanFinished(execution)),
+          2 * 60 * 1000
+        )
+      ).catch(error => {
+        dispatch(
+          stopSubmit('newPlanForm', {
+            _error: {
+              title: formatMessage(messages.planCreationFailed),
+              message: error.message
+            }
+          })
+        );
+      });
+    };
+  },
+
   createPlanFinished(execution) {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
@@ -372,7 +404,8 @@ export default {
         state
       } = execution;
       if (state === 'SUCCESS') {
-        dispatch(this.createPlanSuccess());
+        dispatch(stopSubmit('newPlanForm'));
+        dispatch(this.createPlanSuccess(planName));
         dispatch(
           NotificationActions.notify({
             type: 'success',
@@ -382,11 +415,15 @@ export default {
             })
           })
         );
-        dispatch(this.fetchPlans());
         history.push('/plans/manage');
       } else {
         dispatch(
-          this.createPlanFailed([{ title: 'Plan creation failed', message }])
+          stopSubmit('newPlanForm', {
+            _error: {
+              title: formatMessage(messages.planCreationFailed),
+              message: sanitizeMessage(message)
+            }
+          })
         );
       }
     };
@@ -444,75 +481,6 @@ export default {
     };
   },
 
-  deployPlanPending(planName) {
-    return {
-      type: PlansConstants.START_DEPLOYMENT_PENDING,
-      payload: planName
-    };
-  },
-
-  deployPlanSuccess(planName) {
-    return {
-      type: PlansConstants.START_DEPLOYMENT_SUCCESS,
-      payload: planName
-    };
-  },
-
-  deployPlanFailed(planName) {
-    return {
-      type: PlansConstants.START_DEPLOYMENT_FAILED,
-      payload: planName
-    };
-  },
-
-  deployPlan(planName) {
-    return dispatch => {
-      dispatch(this.deployPlanPending(planName));
-      dispatch(
-        startWorkflow(
-          MistralConstants.DEPLOYMENT_DEPLOY_PLAN,
-          {
-            container: planName,
-            timeout: 240
-          },
-          execution => dispatch(this.deployPlanFinished(execution))
-        )
-      )
-        .then(response => {
-          dispatch(StackActions.fetchStacks());
-        })
-        .catch(error => {
-          dispatch(
-            handleErrors(error, `Plan ${planName} could not be deployed`)
-          );
-          dispatch(this.deployPlanFailed(planName));
-        });
-    };
-  },
-
-  deployPlanFinished(execution) {
-    return (dispatch, getState, { getIntl }) => {
-      const { formatMessage } = getIntl(getState());
-      const {
-        input: { container: planName },
-        output: { message },
-        state
-      } = execution;
-      if (state === 'ERROR') {
-        dispatch(this.deployPlanFailed(planName));
-        dispatch(
-          NotificationActions.notify({
-            title: formatMessage(messages.deploymentFailedNotificationTitle),
-            message
-          })
-        );
-      } else {
-        dispatch(this.deployPlanSuccess(planName));
-        dispatch(StackActions.fetchStacks());
-      }
-    };
-  },
-
   exportPlanPending(planName) {
     return {
       type: PlansConstants.EXPORT_PLAN_PENDING,
@@ -565,7 +533,7 @@ export default {
         dispatch(
           NotificationActions.notify({
             title: formatMessage(messages.exportFailedNotificationTitle),
-            message
+            message: sanitizeMessage(message)
           })
         );
       } else {
@@ -591,11 +559,7 @@ export const uploadFilesToContainer = (container, files) => dispatch =>
   when.all(
     Object.keys(files).map(fileName =>
       dispatch(
-        SwiftApiService.createObject(
-          container,
-          fileName,
-          files[fileName].contents
-        )
+        SwiftApiService.createObject(container, fileName, files[fileName])
       )
     )
   );
